@@ -60,8 +60,9 @@ _dbm_dbm_get(dbmobject *self, PyObject *args)
     PyObject *default_value = NULL;
 
     if (!PyArg_ParseTuple(args, "s#|O:get",
-        &key, &key_length, &default_value))
+        &key, &key_length, &default_value)) {
         goto exit;
+    }
     return_value = _dbm_dbm_get_impl(self, key, key_length, default_value);
 
 exit:
@@ -93,8 +94,9 @@ _dbm_dbm_setdefault(dbmobject *self, PyObject *args)
     PyObject *default_value = NULL;
 
     if (!PyArg_ParseTuple(args, "s#|O:setdefault",
-        &key, &key_length, &default_value))
+        &key, &key_length, &default_value)) {
         goto exit;
+    }
     return_value = _dbm_dbm_setdefault_impl(self, key, key_length, default_value);
 
 exit:
@@ -131,11 +133,12 @@ dbmopen(PyObject *module, PyObject *args)
     int mode = 438;
 
     if (!PyArg_ParseTuple(args, "s|si:open",
-        &filename, &flags, &mode))
+        &filename, &flags, &mode)) {
         goto exit;
+    }
     return_value = dbmopen_impl(module, filename, flags, mode);
 
 exit:
     return return_value;
 }
-/*[clinic end generated code: output=fff12f168cdf8b43 input=a9049054013a1b77]*/
+/*[clinic end generated code: output=99adf966ef0475ff input=a9049054013a1b77]*/
